      SUBROUTINE ALD1R(G,MDG,M,N,H,X,XNORM,W,INDEX,IERR)
C
C*******************************************************
C
C    PURPOSE: **** LEAST DISTANCE PROGRAMMING **********
C
C  GIVEN AN M BY N  MATRIX  .G. AND  AN  M-VECTOR .H.
C  COMPUTE  AN  N-VECTOR .X.  WHICH SOLVES THE LEAST
C  SQUARES PROBLEM:
C          G * X >= H     SUBJECT TO //X// - MIN
C
C  SOURCE:LAWSON C.L.,HANSON R.J. 'SOLVING LEAST SQUARES
C  PROBLEM' , PRENTICE-HALL INC.,ENGLEWOOD CLIFFS,NEW
C  JERSEY, 1974.
C
C    PARAMETERS:
C G,MDG,M,N - MDG IS THE FIRST DIMENSIONING PARAMETER FOR
C             THE ARRAY .G( ).  ON ENTRY G( ) CONTAINS THE
C             M BY N MATRIX .G.
C H         - ON ENTRY H CONTAINS THE M-VECTOR H.
C X         - ON ENTRY X NEED NOT BE INITIALIZED.
C             ON EXIT X WILL CONTAIN THE SOLUTION VECTOR
C             (IF IERR=0).
C XNORM     - ON EXIT RNORM CONTAINS THE EUCLIDEAN NORM OF
C             THE SOLUTION VECTOR X.
C W( )      - AN (N+1)*(M+2)+2*M - ARRAY OF WORKING SPACE.
C INDEX( )  - AN INTEGER WORKING ARRAY OF LEANGTH AT
C             LEAST M.
C IERR      - THIS IS A DIAGNOSTICAL PARAMETER USED AS A
C             SUCCESS-FAILURE FLAG WITH  MEANINGS:
C              0 - THE SOLUTION HAS BEEN COMPUTED
C                  SUCCESSFULLY.
C              1 - THE SYSTEM HAS NO SOLUTION.
C             65 - THE DIMENSIONS OF THE PROBLEM ARE BAD.
C  EXTERNAL SUBROUTINES:
C           ALN1R  (NONNEGATIVE LIST SQUARES),
C           ALN1R4 (COMPUTE DIFFERENCE DIFF=X-Y)
C
C*********************************************************
C
      INTEGER   INDEX(M)
      DIMENSION G(MDG,N),H(M),X(N),W(1)
      DATA ZERO/0.0E0/,ONE/1.0E0/
      IF (N.LE.0) GO TO 120
      DO 10 J=1,N
   10 X(J)=ZERO
      XNORM=ZERO
      IF(M.LE.0) GO TO 110
C
C    THE DECLARED DIMENSION OF W MUST BE AT LEAST
C               (N+1)*(M+2)+2*M
C
C   FIRST (N+1)*M LOCKS OF W = MATRIX E FOR PROBLEM NNLS
C    NEXT     N+1 LOCKS OF W = VECTOR F FOR PROBLEM NNLS
C    NEXT     N+1 LOCKS OF W = VECTOR Z FOR PROBLEM NNLS
C    NEXT       M LOCKS OF W = VECTOR Y FOR PROBLEM NNLS
C    NEXT       M LOCKS OF W = VECTOR WDUAL FOR NNLS
C     COPY G**T INTO FIRST N ROWS AND M COLUMNS OF E
C     COPY H**T INTO ROW N+1 OF E
C
      IW=0
      DO 30 J=1,M
      DO 20 I=1,N
      IW=IW+1
   20 W(IW)=G(J,I)
      IW=IW+1
   30 W(IW)=H(J)
      IF=IW+1
C     STORE N ZEROS FOLLOWED BY A ONE INTO F
      DO 40 I=1,N
      IW=IW+1
      W(IW)=ZERO
   40 CONTINUE
      W(IW+1)=ONE
C
      NP1=N+1
      IZ=IW+2
      IY=IZ+NP1
      IWDUAL=IY+M
C
      CALL ALN1R(W,NP1,NP1,M,W(IF),W(IY),RNORM,W(IWDUAL),
     *           W(IZ),INDEX,IERR)
C     USE THE FOLLOWING RETURN IF UNSUCCESSFUL IN NNLS
      IF(IERR.NE.0) RETURN
      IF(RNORM) 130,130,50
   50 FAC=ONE
      IW=IY-1
      DO 60 I=1,M
      IW=IW+1
C     HERE WE ARE USING THE SOLUTION VECTOR Y
      FAC=FAC-H(I)*W(IW)
   60 CONTINUE
C
      IF(ALN1R4(ONE+FAC,ONE)) 130,130,70
   70 FAC=ONE/FAC
      DO 90 J=1,N
      IW=IY-1
      DO 80 I=1,M
      IW=IW+1
C     HERE WE ARE USING THE SOLUTION VECTOR Y
      X(J)=X(J)+G(I,J)*W(IW)
   80 CONTINUE
      X(J)=X(J)*FAC
   90 CONTINUE
      DO 100 J=1,N
      XNORM=XNORM+X(J)**2
  100 CONTINUE
      XNORM=SQRT(XNORM)
C     SUCCESSFUL RETURN
  110 IERR=0
      RETURN
C     ERROR RETURN      N.LE.0
  120 IERR=65
      RETURN
C     RETURNING WITH CONSTRAINS NOT COMPATIBLE
  130 IERR=1
      RETURN
      END
