      SUBROUTINE ALN1R1(MODE,LPIVOT,L1,M,U,IUE,UP,C,ICE,ICV,NCV)
C
C*********************************************************
C
C    PURPOSE:
C       CONSTRUCTION AND/OR APPLICATION OF A SINGLE
C       HOUSEHOLDER TRANSFORMATION
C
C    REFERENCES:
C   1. GOLUB G.H., AND VAN LOAN CH.F. MATRIX
C      COMPUTATIONS.-JOHNS HOPKINS UNIV.PRESS, 1983
C   2. LAWSON C.L., AND HANSON R.J. SOLVING LEAST SQUARES
C      PROBLEMS. - PRENTICE-HALL,INC.,ENGLEWOOD CLIFFS,
C      NEW JERSEY, 1974
C
C    USAGE:
C   CALL ALN1R1 (MODE,LPIVOT,L1,M,U,IUE,UP,C,ICE,ICV,NCV)
C
C    PARAMETERS:
C
C    MODE - INTEGER VARIABLE - A CONTROL PARAMETER TO
C           SELECT ALGORITHM H1 (CONSTRUCTION AND APPLI-
C           CATION OF Q) OR H2 (APPLICATION ONLY) [2];
C  LPIVOT - INTEGER VARIABLE - IS THE INDEX OF PIVOT
C           ENTRY LP ;
C      L1 - INTEGER VARIABLE - IS DESCRIBED ABOVE;
C           IF L1 <= M, THE TRANSFORMATION WILL BE
C           CONSTRUCTED TO ZERO ELEMENTS INDEXED FROM L1
C           THROUGH M. IF L1 > M  THE SUBROUTINE DOES
C           AN IDENTITY TRANSFORMATION;
C      M - INTEGER VARIABLE - THE SIZE OF PROBLEM (THE
C          LENGTH OF VECTOR V DESCRIBED ABOVE);
C      U - REAL ARRAY OF SIZE IUE BY M - ON ENTRANCE
C          TO H1 CONTAINS THE PIVOT VECTOR V WITH THE
C          STORAGE INCREMENT IUE BETWEEN ELEMENTS;
C          ON EXIT FROM H1 U AND UP CONTAIN QUANTITIES
C          DEFINING THE VECTOR U OF THE HOUSEHOLDER
C          TRANSFORMATION (SEE (4));
C          ON ENTRY TO H2 U AND UP SHOULD CONTAIN QUANTI-
C          TIES PREVIOUSLY COMPUTED BY H1. THESE WILL NOT
C          BE MODIFIED BY H2;
C   IUE - INTEGER VARIABLE - IS THE STORAGE INCREMENT
C         BETWEEN ELEMENTS OF U;
C    UP - REAL VARIABLE - ON ENTRANCE TO H1 IS ARBITRARY;
C         ON EXIT FROM H1 CONTAINS U(LP) (SEE (4));
C         ON ENTRANCE TO H2 UP SHOULD CONTAIN U(LP) PRE-
C         VIOUSLY COMPUTED BY H1 AND WILL NOT CHANGE BY
C         H2;
C     C - REAL VECTOR OF LENGTH  ICV*NCV  AT LEAST IF
C         EL = Q*F MUST BE CALCULATED AND OF LENGTH
C         ICE*M  AT LEAST IF  ER = F*Q  MUST BE CALCU-
C         LATED  -  ON ENTRY TO H1 OR H2  C  CONTAINS A
C         MATRIX WHICH BE REGARDED AS A SET OF VECTORS
C         TO WHICH THE HOUSEHOLDER TRANSFORMATION IS TO
C         BE APPLIED. ON EXIT  C  CONTAINS THE SET OF
C         TRANSFORMED VECTORS;
C   ICE - INTEGER VARIABLE - STORAGE INCREMENT BETWEEN
C         ELEMENTS OF VECTORS IN C;
C   ICV - INTEGER VARIABLE - STORAGE INCREMENT BETWEEN
C         VECTORS IN  C;
C   NCV - INTEGER VARIABLE - NUMBER OF VECTORS IN  C  TO
C         BE TRANSFORMED. IF  NCV <= 0 NO OPERATIONS
C         WILL BE DONE ON  C .
C
C*********************************************************
C
      INTEGER ICE,ICV,IUE,LPIVOT,L1,M,MODE,NCV
      INTEGER I,INCR,I2,I3,I4,J
      REAL C,U,UP
      REAL CL,CLINV,ONE,SM1,SYS056
      DOUBLE PRECISION B,DONE,SM
      DIMENSION C(1),U(IUE,M)
      DATA DONE /1.D0/, ONE /1.E0/
C     SYS056 = SYS055
      DATA  SYS056 /1.175495E-38/
C
C       SYS056 IS INVERSE VALUE OF MAXIMAL REAL TYPE
C       NUMBER WHICH MAY BE REPRESENTED AT THE IBM PC .
C
      IF(0.GE.LPIVOT.OR.LPIVOT.GE.L1.OR.L1.GT.M) GO TO 170
      CL=ABS(U(1,LPIVOT))
      IF(MODE.EQ.2) GO TO 100
C
C     .....CONSTRUCT THE TRANSFORMATION.........
C
      DO 10 J=L1,M
   10 CL=AMAX1(ABS(U(1,J)),CL)
      IF(CL) 170,170,20
   20 IF(CL-SYS056) 30,30,50
   30 SM=(DBLE(U(1,LPIVOT))/CL)**2
      DO 40 J=L1,M
   40 SM=SM+(DBLE (U(1,J))/CL)**2
      GO TO 70
   50 CLINV=ONE/CL
      SM=(DBLE(U(1,LPIVOT))*CLINV)**2
      DO 60 J=L1,M
   60 SM=SM+(DBLE(U(1,J))*CLINV)**2
C
C     CONVERT DOUBLE PRECISION .SM.
C     TO SINGLE PRECISION .SM1.
C
   70 SM1=SNGL(DSQRT(SM))
      CL=CL*SM1
      IF(U(1,LPIVOT)) 90,90,80
   80 CL=-CL
   90 UP=U(1,LPIVOT)-CL
      U(1,LPIVOT)=CL
      GO TO 110
C
C     APPLY THE TRANSFORMATION  1+U*TRANSPOSE(U)/B TO C*
C
  100 IF(CL) 170,170,110
  110 IF(NCV.LE.0) GO TO 170
      B=DBLE(UP)*U(1,LPIVOT)
C
C     B MUST BE NONPOSITIVE HERE. IF B=0. RETURN
C
      IF(B) 120,170,170
  120 B=DONE/B
      I2=1-ICV+ICE*(LPIVOT-1)
      INCR=ICE*(L1-LPIVOT)
      DO 160 J=1,NCV
      I2=I2+ICV
      I3=I2+INCR
      I4=I3
      SM=C(I2)*DBLE(UP)
      DO 130 I=L1,M
      SM=SM+C(I3)*DBLE(U(1,I))
  130 I3=I3+ICE
      IF(SM) 140,160,140
  140 SM=SM*B
      C(I2)=C(I2)+SM*DBLE(UP)
      DO 150 I=L1,M
      C(I4)=C(I4)+SM*DBLE(U(1,I))
  150 I4=I4+ICE
  160 CONTINUE
  170 RETURN
      END
